/*
 * Copyright (C) 2009 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copied from BlueToothChat example, modified to only list paired
 * devices and remove debug code.
 */

package my.lan.zooty;

import java.util.Set;

import android.app.Activity;
import android.util.Log;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;

/**
 * This Activity appears as a dialog. It lists any paired devices and
 * devices detected in the area after discovery. When a device is chosen
 * by the user, the MAC address of the device is sent back to the parent
 * Activity in the result Intent.
 */
public class DeviceListActivity extends Activity {
    // Debugging
    private static final String TAG = "DeviceListActivity";
    private static final boolean D = true;

    // Return Intent extra
    public static String EXTRA_DEVICE_ADDRESS = "device_address";

    // Member fields
    private BluetoothAdapter mBtAdapter;
    private ArrayAdapter<String> mPairedDevicesArrayAdapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (D) Log.d(TAG, "onCreate 1");

        // Setup the window
        requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
        setContentView(R.layout.device_list);

        if (D) Log.d(TAG, "onCreate 2");
        // Set result CANCELED incase the user backs out
        setResult(Activity.RESULT_CANCELED);

        if (D) Log.d(TAG, "onCreate 3");
        // Initialize array adapter.
        mPairedDevicesArrayAdapter =
           new ArrayAdapter<String>(this, R.layout.device_name);

        if (D) Log.d(TAG, "onCreate 4");
        // Find and set up the ListView for paired devices
        ListView pairedListView = (ListView) findViewById(R.id.paired_devices);
        pairedListView.setAdapter(mPairedDevicesArrayAdapter);
        pairedListView.setOnItemClickListener(mDeviceClickListener);

        if (D) Log.d(TAG, "onCreate 5");
        // Get the local Bluetooth adapter
        mBtAdapter = BluetoothAdapter.getDefaultAdapter();

        if (mBtAdapter == null) {
           finish();
        } else {
           if (D) Log.d(TAG, "onCreate 6");
           // Get a set of currently paired devices
           Set<BluetoothDevice> pairedDevices = mBtAdapter.getBondedDevices();

           if (D) Log.d(TAG, "onCreate 7");
           // If there are paired devices, add each one to the ArrayAdapter
           if (pairedDevices.size() > 0) {
               findViewById(R.id.title_paired_devices).setVisibility(View.VISIBLE);
               for (BluetoothDevice device : pairedDevices) {
                   mPairedDevicesArrayAdapter.add(device.getName() + "\n" + device.getAddress());
               }
           } else {
               String noDevices = getResources().getText(R.string.none_paired).toString();
               mPairedDevicesArrayAdapter.add(noDevices);
           }
        }
    }

    // The on-click listener for all devices in the ListViews
    private OnItemClickListener mDeviceClickListener = new OnItemClickListener() {
        public void onItemClick(AdapterView<?> av, View v, int arg2, long arg3) {

            // Get the device MAC address, which is the last 17 chars in the View
            String info = ((TextView) v).getText().toString();
            String address = info.substring(info.length() - 17);

            // Create the result Intent and include the MAC address
            Intent intent = new Intent();
            intent.putExtra(EXTRA_DEVICE_ADDRESS, address);

            // Set result and finish this Activity
            setResult(Activity.RESULT_OK, intent);
            finish();
        }
    };
}
